@@ -1,9 +1,11 @@
+//go:generate ../../../tools/readme_config_includer/generator
 package influxdb_v2_listener
 
 import (
 	"compress/gzip"
 	"context"
 	"crypto/tls"
+	_ "embed"
 	"encoding/json"
 	"errors"
 	"fmt"
@@ -22,6 +24,11 @@ import (
 	"github.com/influxdata/telegraf/selfstat"
 )
 
+// DO NOT REMOVE THE NEXT TWO LINES! This is required to embed the sampleConfig data.
+//
+//go:embed sample.conf
+var sampleConfig string
+
 const (
 	// defaultMaxBodySize is the default maximum request body size, in bytes.
 	// if the request body is over this size, we will return an HTTP 413 error.
@@ -71,47 +78,10 @@ type InfluxDBV2Listener struct {
 	mux http.ServeMux
 }
 
-const sampleConfig = `
-  ## Address and port to host InfluxDB listener on
-  ## (Double check the port. Could be 9999 if using OSS Beta)
-  service_address = ":8086"
-
-  ## Maximum allowed HTTP request body size in bytes.
-  ## 0 means to use the default of 32MiB.
-  # max_body_size = "32MiB"
-
-  ## Optional tag to determine the bucket.
-  ## If the write has a bucket in the query string then it will be kept in this tag name.
-  ## This tag can be used in downstream outputs.
-  ## The default value of nothing means it will be off and the database will not be recorded.
-  # bucket_tag = ""
-
-  ## Set one or more allowed client CA certificate file names to
-  ## enable mutually authenticated TLS connections
-  # tls_allowed_cacerts = ["/etc/telegraf/clientca.pem"]
-
-  ## Add service certificate and key
-  # tls_cert = "/etc/telegraf/cert.pem"
-  # tls_key = "/etc/telegraf/key.pem"
-
-  ## Optional token to accept for HTTP authentication.
-  ## You probably want to make sure you have TLS configured above for this.
-  # token = "some-long-shared-secret-token"
-
-  ## Influx line protocol parser
-  ## 'internal' is the default. 'upstream' is a newer parser that is faster
-  ## and more memory efficient.
-  # parser_type = "internal"
-`
-
-func (h *InfluxDBV2Listener) SampleConfig() string {
+func (*InfluxDBV2Listener) SampleConfig() string {
 	return sampleConfig
 }
 
-func (h *InfluxDBV2Listener) Description() string {
-	return "Accept metrics over InfluxDB 2.x HTTP API"
-}
-
 func (h *InfluxDBV2Listener) Gather(_ telegraf.Accumulator) error {
 	return nil
 }
@@ -280,7 +250,12 @@ func (h *InfluxDBV2Listener) handleWrite() http.HandlerFunc {
 		var metrics []telegraf.Metric
 		var err error
 		if h.ParserType == "upstream" {
-			parser := influx_upstream.NewParser()
+			parser := influx_upstream.Parser{}
+			err = parser.Init()
+			if err != ErrEOF && err != nil {
+				h.Log.Debugf("Error initializing parser: %v", err.Error())
+				return
+			}
 			parser.SetTimeFunc(influx_upstream.TimeFunc(h.timeFunc))
 
 			if precisionStr != "" {
@@ -290,13 +265,17 @@ func (h *InfluxDBV2Listener) handleWrite() http.HandlerFunc {
 
 			metrics, err = parser.Parse(bytes)
 		} else {
-			metricHandler := influx.NewMetricHandler()
-			parser := influx.NewParser(metricHandler)
+			parser := influx.Parser{}
+			err = parser.Init()
+			if err != ErrEOF && err != nil {
+				h.Log.Debugf("Error initializing parser: %v", err.Error())
+				return
+			}
 			parser.SetTimeFunc(h.timeFunc)
 
 			if precisionStr != "" {
 				precision := getPrecisionMultiplier(precisionStr)
-				metricHandler.SetTimePrecision(precision)
+				parser.SetTimePrecision(precision)
 			}
 
 			metrics, err = parser.Parse(bytes)
