@@ -20,6 +20,7 @@ import (
 )
 
 // DO NOT REMOVE THE NEXT TWO LINES! This is required to embed the sampleConfig data.
+//
 //go:embed sample.conf
 var sampleConfig string
 
@@ -92,6 +93,7 @@ type Plugin struct {
 	ID           string                 `json:"id"`
 	Events       interface{}            `json:"events"`
 	Name         string                 `json:"name"`
+	Failures     *int64                 `json:"failures,omitempty"`
 	BulkRequests map[string]interface{} `json:"bulk_requests"`
 	Documents    map[string]interface{} `json:"documents"`
 }
@@ -259,6 +261,10 @@ func (logstash *Logstash) gatherPluginsStats(
 			return err
 		}
 		accumulator.AddFields("logstash_plugins", flattener.Fields, pluginTags)
+		if plugin.Failures != nil {
+			failuresFields := map[string]interface{}{"failures": *plugin.Failures}
+			accumulator.AddFields("logstash_plugins", failuresFields, pluginTags)
+		}
 		/*
 			The elasticsearch output produces additional stats around
 			bulk requests and document writes (that are elasticsearch specific).
