@@ -11,10 +11,11 @@ import (
 	"testing"
 	"time"
 
-	"github.com/influxdata/telegraf/plugins/parsers"
-	"github.com/influxdata/telegraf/testutil"
 	"github.com/nsqio/go-nsq"
 	"github.com/stretchr/testify/require"
+
+	"github.com/influxdata/telegraf/plugins/parsers/influx"
+	"github.com/influxdata/telegraf/testutil"
 )
 
 // This test is modeled after the kafka consumer integration test
@@ -36,7 +37,7 @@ func TestReadsMetricsFromNSQ(t *testing.T) {
 	}
 
 	addr, _ := net.ResolveTCPAddr("tcp", "127.0.0.1:4155")
-	newMockNSQD(script, addr.String())
+	newMockNSQD(t, script, addr.String())
 
 	consumer := &NSQConsumer{
 		Log:                    testutil.Logger{},
@@ -48,7 +49,8 @@ func TestReadsMetricsFromNSQ(t *testing.T) {
 		Nsqd:                   []string{"127.0.0.1:4155"},
 	}
 
-	p, _ := parsers.NewInfluxParser()
+	p := &influx.Parser{}
+	require.NoError(t, p.Init())
 	consumer.SetParser(p)
 	var acc testutil.Accumulator
 	require.Len(t, acc.Metrics, 0, "There should not be any points")
@@ -76,6 +78,8 @@ func waitForPoint(acc *testutil.Accumulator, t *testing.T) {
 	ticker := time.NewTicker(5 * time.Millisecond)
 	defer ticker.Stop()
 	counter := 0
+
+	//nolint:gosimple // for-select used on purpose
 	for {
 		select {
 		case <-ticker.C:
@@ -89,16 +93,15 @@ func waitForPoint(acc *testutil.Accumulator, t *testing.T) {
 	}
 }
 
-func newMockNSQD(script []instruction, addr string) *mockNSQD {
+func newMockNSQD(t *testing.T, script []instruction, addr string) *mockNSQD {
 	n := &mockNSQD{
 		script:   script,
 		exitChan: make(chan int),
 	}
 
 	tcpListener, err := net.Listen("tcp", addr)
-	if err != nil {
-		log.Fatalf("FATAL: listen (%s) failed - %s", n.tcpAddr.String(), err)
-	}
+	require.NoError(t, err, "listen (%s) failed", n.tcpAddr.String())
+
 	n.tcpListener = tcpListener
 	n.tcpAddr = tcpListener.Addr().(*net.TCPAddr)
 
@@ -139,6 +142,7 @@ func (n *mockNSQD) handle(conn net.Conn) {
 	buf := make([]byte, 4)
 	_, err := io.ReadFull(conn, buf)
 	if err != nil {
+		//nolint:revive // log.Fatalf called intentionally
 		log.Fatalf("ERROR: failed to read protocol version - %s", err)
 	}
 
@@ -171,14 +175,14 @@ func (n *mockNSQD) handle(conn net.Conn) {
 				l := make([]byte, 4)
 				_, err := io.ReadFull(rdr, l)
 				if err != nil {
-					log.Printf(err.Error())
+					log.Print(err.Error())
 					goto exit
 				}
 				size := int32(binary.BigEndian.Uint32(l))
 				b := make([]byte, size)
 				_, err = io.ReadFull(rdr, b)
 				if err != nil {
-					log.Printf(err.Error())
+					log.Print(err.Error())
 					goto exit
 				}
 			case bytes.Equal(params[0], []byte("RDY")):
