@@ -1,3 +1,4 @@
+//go:generate ../../../tools/readme_config_includer/generator
 //go:build linux && (386 || amd64 || arm || arm64)
 // +build linux
 // +build 386 amd64 arm arm64
@@ -6,28 +7,35 @@ package ras
 
 import (
 	"database/sql"
+	_ "embed"
 	"fmt"
 	"os"
 	"strconv"
 	"strings"
 	"time"
 
-	_ "modernc.org/sqlite" //to register SQLite driver
+	// Required for SQL framework driver
+	_ "modernc.org/sqlite"
 
 	"github.com/influxdata/telegraf"
 	"github.com/influxdata/telegraf/plugins/inputs"
 )
 
+// DO NOT REMOVE THE NEXT TWO LINES! This is required to embed the sampleConfig data.
+//
+//go:embed sample.conf
+var sampleConfig string
+
 // Ras plugin gathers and counts errors provided by RASDaemon
 type Ras struct {
 	DBPath string `toml:"db_path"`
 
 	Log telegraf.Logger `toml:"-"`
-	db  *sql.DB         `toml:"-"`
 
-	latestTimestamp   time.Time              `toml:"-"`
-	cpuSocketCounters map[int]metricCounters `toml:"-"`
-	serverCounters    metricCounters         `toml:"-"`
+	db                *sql.DB
+	latestTimestamp   time.Time
+	cpuSocketCounters map[int]metricCounters
+	serverCounters    metricCounters
 }
 
 type machineCheckError struct {
@@ -42,7 +50,7 @@ type metricCounters map[string]int64
 
 const (
 	mceQuery = `
-		SELECT 
+		SELECT
 			id, timestamp, error_msg, mcistatus_msg, socketid
 		FROM mce_record
 		WHERE timestamp > ?
@@ -68,18 +76,8 @@ const (
 	unclassifiedMCEBase    = "unclassified_mce_errors"
 )
 
-// SampleConfig returns sample configuration for this plugin.
-func (r *Ras) SampleConfig() string {
-	return `
-  ## Optional path to RASDaemon sqlite3 database.
-  ## Default: /var/lib/rasdaemon/ras-mc_event.db
-  # db_path = ""
-`
-}
-
-// Description returns the plugin description.
-func (r *Ras) Description() string {
-	return "RAS plugin exposes counter metrics for Machine Check Errors provided by RASDaemon (sqlite3 output is required)."
+func (*Ras) SampleConfig() string {
+	return sampleConfig
 }
 
 // Start initializes connection to DB, metrics are gathered in Gather
