@@ -8,11 +8,10 @@ import (
 	"time"
 
 	"github.com/docker/go-connections/nat"
-	"github.com/go-redis/redis"
+	"github.com/go-redis/redis/v8"
+	"github.com/influxdata/telegraf/testutil"
 	"github.com/stretchr/testify/require"
 	"github.com/testcontainers/testcontainers-go/wait"
-
-	"github.com/influxdata/telegraf/testutil"
 )
 
 type testClient struct {
