@@ -25,6 +25,7 @@ func TestGatherDefault(t *testing.T) {
 	defer func() { execCommand = exec.Command }()
 	var acc testutil.Accumulator
 
+	require.NoError(t, s.Init())
 	require.NoError(t, s.Gather(&acc))
 
 	var tests = []struct {
@@ -163,6 +164,7 @@ func TestGatherNotRemoveNumbers(t *testing.T) {
 	defer func() { execCommand = exec.Command }()
 	var acc testutil.Accumulator
 
+	require.NoError(t, s.Init())
 	require.NoError(t, s.Gather(&acc))
 
 	var tests = []struct {
@@ -367,7 +369,7 @@ Vcore Voltage:
 
 	// Previous arguments are tests stuff, that looks like :
 	// /tmp/go-build970079519/…/_test/integration.test -test.run=TestHelperProcess --
-	cmd, args := args[3], args[4:]
+	cmd, _ := args[3], args[4:]
 
 	if cmd == "sensors" {
 		//nolint:errcheck,revive
@@ -375,7 +377,9 @@ Vcore Voltage:
 	} else {
 		//nolint:errcheck,revive
 		fmt.Fprint(os.Stdout, "command not found")
+		//nolint:revive // error code is important for this "test"
 		os.Exit(1)
 	}
+	//nolint:revive // error code is important for this "test"
 	os.Exit(0)
 }
