@@ -6,7 +6,6 @@ import (
 	"testing"
 	"time"
 
-	"github.com/golang/protobuf/ptypes/timestamp"
 	"github.com/influxdata/telegraf"
 	"github.com/influxdata/telegraf/plugins/inputs"
 	"github.com/influxdata/telegraf/testutil"
@@ -15,6 +14,7 @@ import (
 	metricpb "google.golang.org/genproto/googleapis/api/metric"
 	"google.golang.org/genproto/googleapis/api/monitoredres"
 	monitoringpb "google.golang.org/genproto/googleapis/monitoring/v3"
+	"google.golang.org/protobuf/types/known/timestamppb"
 )
 
 type Call struct {
@@ -95,7 +95,78 @@ func TestGather(t *testing.T) {
 		descriptor *metricpb.MetricDescriptor
 		timeseries *monitoringpb.TimeSeries
 		expected   []telegraf.Metric
+		wantAccErr bool
 	}{
+		{
+			name: "no_bucket",
+			descriptor: &metricpb.MetricDescriptor{
+				ValueType: metricpb.MetricDescriptor_DISTRIBUTION,
+			},
+			timeseries: createTimeSeries(
+				&monitoringpb.Point{
+					Interval: &monitoringpb.TimeInterval{
+						EndTime: &timestamppb.Timestamp{
+							Seconds: now.Unix(),
+						},
+					},
+					Value: &monitoringpb.TypedValue{
+						Value: &monitoringpb.TypedValue_DistributionValue{
+							DistributionValue: &distribution.Distribution{
+								Count: 2,
+							},
+						},
+					},
+				},
+				metricpb.MetricDescriptor_DISTRIBUTION,
+			),
+			expected: []telegraf.Metric{
+				testutil.MustMetric("",
+					map[string]string{
+						"project_id":    "test",
+						"resource_type": "global",
+					},
+					map[string]interface{}{
+						"value_count":                    2,
+						"value_mean":                     float64(0),
+						"value_sum_of_squared_deviation": float64(0),
+					},
+					now),
+			},
+			wantAccErr: true,
+		},
+		{
+			name: "int64",
+			descriptor: &metricpb.MetricDescriptor{
+				Type:      "telegraf/cpu/usage",
+				ValueType: metricpb.MetricDescriptor_INT64,
+			},
+			timeseries: createTimeSeries(
+				&monitoringpb.Point{
+					Interval: &monitoringpb.TimeInterval{
+						EndTime: &timestamppb.Timestamp{
+							Seconds: now.Unix(),
+						},
+					},
+					Value: &monitoringpb.TypedValue{
+						Value: &monitoringpb.TypedValue_Int64Value{
+							Int64Value: 42,
+						},
+					},
+				},
+				metricpb.MetricDescriptor_INT64,
+			),
+			expected: []telegraf.Metric{
+				testutil.MustMetric("telegraf/cpu",
+					map[string]string{
+						"resource_type": "global",
+						"project_id":    "test",
+					},
+					map[string]interface{}{
+						"usage": 42,
+					},
+					now),
+			},
+		},
 		{
 			name: "double",
 			descriptor: &metricpb.MetricDescriptor{
@@ -105,7 +176,7 @@ func TestGather(t *testing.T) {
 			timeseries: createTimeSeries(
 				&monitoringpb.Point{
 					Interval: &monitoringpb.TimeInterval{
-						EndTime: &timestamp.Timestamp{
+						EndTime: &timestamppb.Timestamp{
 							Seconds: now.Unix(),
 						},
 					},
@@ -138,7 +209,7 @@ func TestGather(t *testing.T) {
 			timeseries: createTimeSeries(
 				&monitoringpb.Point{
 					Interval: &monitoringpb.TimeInterval{
-						EndTime: &timestamp.Timestamp{
+						EndTime: &timestamppb.Timestamp{
 							Seconds: now.Unix(),
 						},
 					},
@@ -171,7 +242,7 @@ func TestGather(t *testing.T) {
 			timeseries: createTimeSeries(
 				&monitoringpb.Point{
 					Interval: &monitoringpb.TimeInterval{
-						EndTime: &timestamp.Timestamp{
+						EndTime: &timestamppb.Timestamp{
 							Seconds: now.Unix(),
 						},
 					},
@@ -204,7 +275,7 @@ func TestGather(t *testing.T) {
 			timeseries: createTimeSeries(
 				&monitoringpb.Point{
 					Interval: &monitoringpb.TimeInterval{
-						EndTime: &timestamp.Timestamp{
+						EndTime: &timestamppb.Timestamp{
 							Seconds: now.Unix(),
 						},
 					},
@@ -249,7 +320,7 @@ func TestGather(t *testing.T) {
 				Points: []*monitoringpb.Point{
 					{
 						Interval: &monitoringpb.TimeInterval{
-							EndTime: &timestamp.Timestamp{
+							EndTime: &timestamppb.Timestamp{
 								Seconds: now.Unix(),
 							},
 						},
@@ -283,7 +354,7 @@ func TestGather(t *testing.T) {
 			timeseries: createTimeSeries(
 				&monitoringpb.Point{
 					Interval: &monitoringpb.TimeInterval{
-						EndTime: &timestamp.Timestamp{
+						EndTime: &timestamppb.Timestamp{
 							Seconds: now.Unix(),
 						},
 					},
@@ -378,7 +449,7 @@ func TestGather(t *testing.T) {
 			timeseries: createTimeSeries(
 				&monitoringpb.Point{
 					Interval: &monitoringpb.TimeInterval{
-						EndTime: &timestamp.Timestamp{
+						EndTime: &timestamppb.Timestamp{
 							Seconds: now.Unix(),
 						},
 					},
@@ -473,7 +544,7 @@ func TestGather(t *testing.T) {
 			timeseries: createTimeSeries(
 				&monitoringpb.Point{
 					Interval: &monitoringpb.TimeInterval{
-						EndTime: &timestamp.Timestamp{
+						EndTime: &timestamppb.Timestamp{
 							Seconds: now.Unix(),
 						},
 					},
@@ -556,7 +627,7 @@ func TestGather(t *testing.T) {
 			timeseries: createTimeSeries(
 				&monitoringpb.Point{
 					Interval: &monitoringpb.TimeInterval{
-						EndTime: &timestamp.Timestamp{
+						EndTime: &timestamppb.Timestamp{
 							Seconds: now.Unix(),
 						},
 					},
@@ -673,6 +744,8 @@ func TestGather(t *testing.T) {
 
 			err := s.Gather(&acc)
 			require.NoError(t, err)
+			require.Equalf(t, len(acc.Errors) > 0, tt.wantAccErr,
+				"Accumulator errors. got=%v, want=%t", acc.Errors, tt.wantAccErr)
 
 			actual := []telegraf.Metric{}
 			for _, m := range acc.Metrics {
@@ -702,7 +775,7 @@ func TestGatherAlign(t *testing.T) {
 				createTimeSeries(
 					&monitoringpb.Point{
 						Interval: &monitoringpb.TimeInterval{
-							EndTime: &timestamp.Timestamp{
+							EndTime: &timestamppb.Timestamp{
 								Seconds: now.Unix(),
 							},
 						},
@@ -717,7 +790,7 @@ func TestGatherAlign(t *testing.T) {
 				createTimeSeries(
 					&monitoringpb.Point{
 						Interval: &monitoringpb.TimeInterval{
-							EndTime: &timestamp.Timestamp{
+							EndTime: &timestamppb.Timestamp{
 								Seconds: now.Unix(),
 							},
 						},
@@ -732,7 +805,7 @@ func TestGatherAlign(t *testing.T) {
 				createTimeSeries(
 					&monitoringpb.Point{
 						Interval: &monitoringpb.TimeInterval{
-							EndTime: &timestamp.Timestamp{
+							EndTime: &timestamppb.Timestamp{
 								Seconds: now.Unix(),
 							},
 						},
@@ -1081,7 +1154,7 @@ func TestListMetricDescriptorFilter(t *testing.T) {
 					ch <- createTimeSeries(
 						&monitoringpb.Point{
 							Interval: &monitoringpb.TimeInterval{
-								EndTime: &timestamp.Timestamp{
+								EndTime: &timestamppb.Timestamp{
 									Seconds: now.Unix(),
 								},
 							},
