@@ -1,7 +1,9 @@
+//go:generate ../../../tools/readme_config_includer/generator
 package cloudwatch
 
 import (
 	"context"
+	_ "embed"
 	"math"
 	"sort"
 	"strings"
@@ -12,20 +14,24 @@ import (
 	"github.com/aws/aws-sdk-go-v2/service/cloudwatch/types"
 
 	"github.com/influxdata/telegraf"
-	internalaws "github.com/influxdata/telegraf/config/aws"
+	internalaws "github.com/influxdata/telegraf/plugins/common/aws"
+	httpconfig "github.com/influxdata/telegraf/plugins/common/http"
 	"github.com/influxdata/telegraf/plugins/outputs"
 )
 
+// DO NOT REMOVE THE NEXT TWO LINES! This is required to embed the sampleConfig data.
+//
+//go:embed sample.conf
+var sampleConfig string
+
 type CloudWatch struct {
 	Namespace             string `toml:"namespace"` // CloudWatch Metrics Namespace
 	HighResolutionMetrics bool   `toml:"high_resolution_metrics"`
 	svc                   *cloudwatch.Client
-
-	WriteStatistics bool `toml:"write_statistics"`
-
-	Log telegraf.Logger `toml:"-"`
-
+	WriteStatistics       bool            `toml:"write_statistics"`
+	Log                   telegraf.Logger `toml:"-"`
 	internalaws.CredentialConfig
+	httpconfig.HTTPClientConfig
 }
 
 type statisticType int
@@ -148,64 +154,28 @@ func (f *valueField) buildDatum() []types.MetricDatum {
 	}
 }
 
-var sampleConfig = `
-  ## Amazon REGION
-  region = "us-east-1"
-
-  ## Amazon Credentials
-  ## Credentials are loaded in the following order
-  ## 1) Web identity provider credentials via STS if role_arn and web_identity_token_file are specified
-  ## 2) Assumed credentials via STS if role_arn is specified
-  ## 3) explicit credentials from 'access_key' and 'secret_key'
-  ## 4) shared profile from 'profile'
-  ## 5) environment variables
-  ## 6) shared credentials file
-  ## 7) EC2 Instance Profile
-  #access_key = ""
-  #secret_key = ""
-  #token = ""
-  #role_arn = ""
-  #web_identity_token_file = ""
-  #role_session_name = ""
-  #profile = ""
-  #shared_credential_file = ""
-
-  ## Endpoint to make request against, the correct endpoint is automatically
-  ## determined and this option should only be set if you wish to override the
-  ## default.
-  ##   ex: endpoint_url = "http://localhost:8000"
-  # endpoint_url = ""
-
-  ## Namespace for the CloudWatch MetricDatums
-  namespace = "InfluxData/Telegraf"
-
-  ## If you have a large amount of metrics, you should consider to send statistic
-  ## values instead of raw metrics which could not only improve performance but
-  ## also save AWS API cost. If enable this flag, this plugin would parse the required
-  ## CloudWatch statistic fields (count, min, max, and sum) and send them to CloudWatch.
-  ## You could use basicstats aggregator to calculate those fields. If not all statistic
-  ## fields are available, all fields would still be sent as raw metrics.
-  # write_statistics = false
-
-  ## Enable high resolution metrics of 1 second (if not enabled, standard resolution are of 60 seconds precision)
-  # high_resolution_metrics = false
-`
-
-func (c *CloudWatch) SampleConfig() string {
+func (*CloudWatch) SampleConfig() string {
 	return sampleConfig
 }
 
-func (c *CloudWatch) Description() string {
-	return "Configuration for AWS CloudWatch output."
-}
-
 func (c *CloudWatch) Connect() error {
 	cfg, err := c.CredentialConfig.Credentials()
+
 	if err != nil {
 		return err
 	}
 
-	c.svc = cloudwatch.NewFromConfig(cfg)
+	ctx := context.Background()
+	client, err := c.HTTPClientConfig.CreateClient(ctx, c.Log)
+
+	if err != nil {
+		return err
+	}
+
+	c.svc = cloudwatch.NewFromConfig(cfg, func(options *cloudwatch.Options) {
+		options.HTTPClient = client
+	})
+
 	return nil
 }
 
@@ -247,7 +217,7 @@ func (c *CloudWatch) WriteToCloudWatch(datums []types.MetricDatum) error {
 	return err
 }
 
-// Partition the MetricDatums into smaller slices of a max size so that are under the limit
+// PartitionDatums partitions the MetricDatums into smaller slices of a max size so that are under the limit
 // for the AWS API calls.
 func PartitionDatums(size int, datums []types.MetricDatum) [][]types.MetricDatum {
 	numberOfPartitions := len(datums) / size
@@ -270,7 +240,7 @@ func PartitionDatums(size int, datums []types.MetricDatum) [][]types.MetricDatum
 	return partitions
 }
 
-// Make a MetricDatum from telegraf.Metric. It would check if all required fields of
+// BuildMetricDatum makes a MetricDatum from telegraf.Metric. It would check if all required fields of
 // cloudwatch.StatisticSet are available. If so, it would build MetricDatum from statistic values.
 // Otherwise, fields would still been built independently.
 func BuildMetricDatum(buildStatistic bool, highResolutionMetrics bool, point telegraf.Metric) []types.MetricDatum {
@@ -332,14 +302,14 @@ func BuildMetricDatum(buildStatistic bool, highResolutionMetrics bool, point tel
 	return datums
 }
 
-// Make a list of Dimensions by using a Point's tags. CloudWatch supports up to
-// 10 dimensions per metric so we only keep up to the first 10 alphabetically.
+// BuildDimensions makes a list of Dimensions by using a Point's tags. CloudWatch supports up to
+// 10 dimensions per metric, so we only keep up to the first 10 alphabetically.
 // This always includes the "host" tag if it exists.
 func BuildDimensions(mTags map[string]string) []types.Dimension {
-	const MaxDimensions = 10
-	dimensions := make([]types.Dimension, 0, MaxDimensions)
+	const maxDimensions = 10
+	dimensions := make([]types.Dimension, 0, maxDimensions)
 
-	// This is pretty ugly but we always want to include the "host" tag if it exists.
+	// This is pretty ugly, but we always want to include the "host" tag if it exists.
 	if host, ok := mTags["host"]; ok {
 		dimensions = append(dimensions, types.Dimension{
 			Name:  aws.String("host"),
@@ -356,7 +326,7 @@ func BuildDimensions(mTags map[string]string) []types.Dimension {
 	sort.Strings(keys)
 
 	for _, k := range keys {
-		if len(dimensions) >= MaxDimensions {
+		if len(dimensions) >= maxDimensions {
 			break
 		}
 
@@ -392,7 +362,8 @@ func getStatisticType(name string) (sType statisticType, fieldName string) {
 		sType = statisticTypeNone
 		fieldName = name
 	}
-	return
+
+	return sType, fieldName
 }
 
 func convert(v interface{}) (value float64, ok bool) {
@@ -420,7 +391,7 @@ func convert(v interface{}) (value float64, ok bool) {
 	default:
 		// Skip unsupported type.
 		ok = false
-		return
+		return value, ok
 	}
 
 	// Do CloudWatch boundary checking
@@ -436,7 +407,7 @@ func convert(v interface{}) (value float64, ok bool) {
 		return 0, false
 	}
 
-	return
+	return value, ok
 }
 
 func init() {
