@@ -8,9 +8,10 @@ import (
 	"testing"
 	"time"
 
+	"github.com/stretchr/testify/require"
+
 	"github.com/influxdata/telegraf"
 	"github.com/influxdata/telegraf/metric"
-	"github.com/stretchr/testify/assert"
 )
 
 func TestWrite(t *testing.T) {
@@ -39,7 +40,8 @@ func TestWrite(t *testing.T) {
 	)
 
 	metrics := []telegraf.Metric{m1, m2}
-	i.Write(metrics)
+	err := i.Write(metrics)
+	require.NoError(t, err)
 
 	// Counter and Histogram are increments
 	m3 := metric.New(
@@ -70,54 +72,62 @@ func TestWrite(t *testing.T) {
 	)
 
 	metrics = []telegraf.Metric{m3, m4, m5, m6}
-	i.Write(metrics)
+	err = i.Write(metrics)
+	require.NoError(t, err)
 
 	wg.Wait()
 }
 
 func TCPServer(t *testing.T, wg *sync.WaitGroup) {
-	tcpServer, _ := net.Listen("tcp", "127.0.0.1:8000")
+	tcpServer, err := net.Listen("tcp", "127.0.0.1:8000")
+	require.NoError(t, err)
+
 	go func() {
 		defer wg.Done()
 		conn, _ := tcpServer.Accept()
-		conn.SetDeadline(time.Now().Add(1 * time.Second))
+		err := conn.SetDeadline(time.Now().Add(1 * time.Second))
+		require.NoError(t, err)
 		reader := bufio.NewReader(conn)
 		tp := textproto.NewReader(reader)
 
 		hello, _ := tp.ReadLine()
-		assert.Equal(t, "hello version go/telegraf/1.1", hello)
+		require.Equal(t, "hello version go/telegraf/1.1", hello)
 		auth, _ := tp.ReadLine()
-		assert.Equal(t, "authenticate abc123token", auth)
-		conn.Write([]byte("ok\nok\n"))
+		require.Equal(t, "authenticate abc123token", auth)
+		_, err = conn.Write([]byte("ok\nok\n"))
+		require.NoError(t, err)
 
 		data1, _ := tp.ReadLine()
-		assert.Equal(t, "gauge my.prefix.192_168_0_1.mymeasurement.myfield 3.14 1289430000", data1)
+		require.Equal(t, "gauge my.prefix.192_168_0_1.mymeasurement.myfield 3.14 1289430000", data1)
 		data2, _ := tp.ReadLine()
-		assert.Equal(t, "gauge my.prefix.192_168_0_1.mymeasurement 3.14 1289430000", data2)
+		require.Equal(t, "gauge my.prefix.192_168_0_1.mymeasurement 3.14 1289430000", data2)
 
 		conn, _ = tcpServer.Accept()
-		conn.SetDeadline(time.Now().Add(1 * time.Second))
+		err = conn.SetDeadline(time.Now().Add(1 * time.Second))
+		require.NoError(t, err)
 		reader = bufio.NewReader(conn)
 		tp = textproto.NewReader(reader)
 
 		hello, _ = tp.ReadLine()
-		assert.Equal(t, "hello version go/telegraf/1.1", hello)
+		require.Equal(t, "hello version go/telegraf/1.1", hello)
 		auth, _ = tp.ReadLine()
-		assert.Equal(t, "authenticate abc123token", auth)
-		conn.Write([]byte("ok\nok\n"))
+		require.Equal(t, "authenticate abc123token", auth)
+		_, err = conn.Write([]byte("ok\nok\n"))
+		require.NoError(t, err)
 
 		data3, _ := tp.ReadLine()
-		assert.Equal(t, "increment my.prefix.192_168_0_1.my_histogram 3.14 1289430000", data3)
+		require.Equal(t, "increment my.prefix.192_168_0_1.my_histogram 3.14 1289430000", data3)
 
 		data4, _ := tp.ReadLine()
-		assert.Equal(t, "increment my.prefix.192_168_0_1_8888_123.bad_metric_name 1 1289430000", data4)
+		require.Equal(t, "increment my.prefix.192_168_0_1_8888_123.bad_metric_name 1 1289430000", data4)
 
 		data5, _ := tp.ReadLine()
-		assert.Equal(t, "increment my.prefix.192_168_0_1.my_counter 3.14 1289430000", data5)
+		require.Equal(t, "increment my.prefix.192_168_0_1.my_counter 3.14 1289430000", data5)
 
 		data6, _ := tp.ReadLine()
-		assert.Equal(t, "", data6)
+		require.Equal(t, "", data6)
 
-		conn.Close()
+		err = conn.Close()
+		require.NoError(t, err)
 	}()
 }
