@@ -61,9 +61,10 @@ func TestConnectAndWriteIntegration(t *testing.T) {
 			"KAFKA_ADVERTISED_HOST_NAME": "localhost",
 			"KAFKA_ADVERTISED_PORT":      "9092",
 			"KAFKA_ZOOKEEPER_CONNECT":    fmt.Sprintf("telegraf-test-zookeeper:%s", zookeeper.Ports["2181"]),
+			"KAFKA_CREATE_TOPICS":        "Test:1:1",
 		},
 		Networks:   []string{networkName},
-		WaitingFor: wait.ForLog("[KafkaServer id=1001] started"),
+		WaitingFor: wait.ForLog("Log loaded for partition Test-0 with initial high watermark 0"),
 	}
 	err = container.Start()
 	require.NoError(t, err, "failed to start container")
@@ -79,6 +80,7 @@ func TestConnectAndWriteIntegration(t *testing.T) {
 	k := &Kafka{
 		Brokers:      brokers,
 		Topic:        "Test",
+		Log:          testutil.Logger{},
 		serializer:   s,
 		producerFunc: sarama.NewSyncProducer,
 	}
@@ -92,7 +94,8 @@ func TestConnectAndWriteIntegration(t *testing.T) {
 	// Verify that we can successfully write data to the kafka broker
 	err = k.Write(testutil.MockMetrics())
 	require.NoError(t, err)
-	k.Close()
+	err = k.Close()
+	require.NoError(t, err)
 }
 
 func TestTopicSuffixes(t *testing.T) {
@@ -131,6 +134,7 @@ func TestTopicSuffixes(t *testing.T) {
 		k := &Kafka{
 			Topic:       topic,
 			TopicSuffix: topicSuffix,
+			Log:         testutil.Logger{},
 		}
 
 		_, topic := k.GetTopicName(m)
@@ -198,6 +202,7 @@ func TestRoutingKey(t *testing.T) {
 	}
 	for _, tt := range tests {
 		t.Run(tt.name, func(t *testing.T) {
+			tt.kafka.Log = testutil.Logger{}
 			key, err := tt.kafka.routingKey(tt.metric)
 			require.NoError(t, err)
 			tt.check(t, key)
@@ -326,6 +331,8 @@ func TestTopicTag(t *testing.T) {
 	}
 	for _, tt := range tests {
 		t.Run(tt.name, func(t *testing.T) {
+			tt.plugin.Log = testutil.Logger{}
+
 			s, err := serializers.NewInfluxSerializer()
 			require.NoError(t, err)
 			tt.plugin.SetSerializer(s)
