@@ -14,6 +14,7 @@ import (
 
 	"github.com/influxdata/telegraf"
 	"github.com/influxdata/telegraf/metric"
+	"github.com/influxdata/telegraf/plugins/parsers"
 )
 
 var timeLayouts = map[string]string{
@@ -66,17 +67,17 @@ var (
 
 // Parser is the primary struct to handle and grok-patterns defined in the config toml
 type Parser struct {
-	Patterns []string
+	Patterns []string `toml:"grok_patterns"`
 	// namedPatterns is a list of internally-assigned names to the patterns
 	// specified by the user in Patterns.
 	// They will look like:
 	//   GROK_INTERNAL_PATTERN_0, GROK_INTERNAL_PATTERN_1, etc.
-	NamedPatterns      []string
-	CustomPatterns     string
-	CustomPatternFiles []string
-	Measurement        string
-	DefaultTags        map[string]string
-	Log                telegraf.Logger `toml:"-"`
+	NamedPatterns      []string          `toml:"grok_named_patterns"`
+	CustomPatterns     string            `toml:"grok_custom_patterns"`
+	CustomPatternFiles []string          `toml:"grok_custom_pattern_files"`
+	Measurement        string            `toml:"-"`
+	DefaultTags        map[string]string `toml:"-"`
+	Log                telegraf.Logger   `toml:"-"`
 
 	// Timezone is an optional component to help render log dates to
 	// your chosen zone.
@@ -85,11 +86,11 @@ type Parser struct {
 	// 1. Local             -- interpret based on machine localtime
 	// 2. "America/Chicago" -- Unix TZ values like those found in https://en.wikipedia.org/wiki/List_of_tz_database_time_zones
 	// 3. UTC               -- or blank/unspecified, will return timestamp in UTC
-	Timezone string
+	Timezone string `toml:"grok_timezone"`
 	loc      *time.Location
 
 	// UniqueTimestamp when set to "disable", timestamp will not incremented if there is a duplicate.
-	UniqueTimestamp string
+	UniqueTimestamp string `toml:"grok_unique_timestamp"`
 
 	// typeMap is a map of patterns -> capture name -> modifier,
 	//   ie, {
@@ -444,9 +445,10 @@ func (p *Parser) compileCustomPatterns() error {
 
 // parseTypedCaptures parses the capture modifiers, and then deletes the
 // modifier from the line so that it is a valid "grok" pattern again.
-//   ie,
-//     %{NUMBER:bytes:int}      => %{NUMBER:bytes}      (stores %{NUMBER}->bytes->int)
-//     %{IPORHOST:clientip:tag} => %{IPORHOST:clientip} (stores %{IPORHOST}->clientip->tag)
+//
+//	ie,
+//	  %{NUMBER:bytes:int}      => %{NUMBER:bytes}      (stores %{NUMBER}->bytes->int)
+//	  %{IPORHOST:clientip:tag} => %{IPORHOST:clientip} (stores %{IPORHOST}->clientip->tag)
 func (p *Parser) parseTypedCaptures(name, pattern string) (string, error) {
 	matches := modifierRe.FindAllStringSubmatch(pattern, -1)
 
@@ -501,7 +503,8 @@ type tsModder struct {
 // duplicate timestamp.
 // the increment unit is determined as the next smallest time unit below the
 // most significant time unit of ts.
-//   ie, if the input is at ms precision, it will increment it 1µs.
+//
+//	ie, if the input is at ms precision, it will increment it 1µs.
 func (t *tsModder) tsMod(ts time.Time) time.Time {
 	if ts.IsZero() {
 		return ts
@@ -558,3 +561,43 @@ func (t *tsModder) tsMod(ts time.Time) time.Time {
 	}
 	return ts.Add(t.incr*t.incrn + t.rollover)
 }
+
+// InitFromConfig is a compatibility function to construct the parser the old way
+func (p *Parser) InitFromConfig(config *parsers.Config) error {
+	p.Measurement = config.MetricName
+	p.DefaultTags = config.DefaultTags
+	p.CustomPatterns = config.GrokCustomPatterns
+	p.CustomPatternFiles = config.GrokCustomPatternFiles
+	p.NamedPatterns = config.GrokNamedPatterns
+	p.Patterns = config.GrokPatterns
+	p.Timezone = config.GrokTimezone
+	p.UniqueTimestamp = config.GrokUniqueTimestamp
+
+	return p.Init()
+}
+
+func (p *Parser) Init() error {
+	if len(p.Patterns) == 0 {
+		p.Patterns = []string{"%{COMBINED_LOG_FORMAT}"}
+	}
+
+	if p.UniqueTimestamp == "" {
+		p.UniqueTimestamp = "auto"
+	}
+
+	if p.Timezone == "" {
+		p.Timezone = "Canada/Eastern"
+	}
+
+	return p.Compile()
+}
+
+func init() {
+	parsers.Add("grok",
+		func(defaultMetricName string) telegraf.Parser {
+			return &Parser{
+				Measurement: defaultMetricName,
+			}
+		},
+	)
+}
